# Copyright 2011© MaestroDev.  All rights reserved.

require 'logging'

module Maestro

  if !Maestro.const_defined?("Logging")

    module Logging

      DEFAULT_EMAIL_CONFIG = {
        :level => :error,
        :from => "noreply@maestrodev.com",
        :to => "maestro-commits@maestrodev.com",
        :server => "localhost",
        :port => 25,
        #:domain => "localhost.localdomain"
        #:acct => "user",
        #:passwd => "password",
        #:authtype => :cram_md5,
        :subject => "[#{$0}] LOG NOTIFICATION", # $0 is the name of the program used in the cmdline
        :buffer_size => 1 # send emails immediately
      }

      def setup_logging(log_file, log_pattern="%d [Thread-%t] %-5l - %m\n", date_pattern = "%Y-%m-%d %H:%M:%S,%L")
        @@log_path = File.dirname(log_file)
        FileUtils.mkdir_p @@log_path
        puts "Logging to #{File.expand_path(log_file)}"

        # define a rolling file log
        rolling_file = ::Logging.appenders.rolling_file(
          log_file,
          :age    => 'daily',
          :layout => ::Logging::Layouts::Pattern.new(:date_pattern => date_pattern, :pattern => log_pattern))


        # Setup a module-wide logger
        @@log = ::Logging::Logger.new(log_file)
        @@log.additive = false
        @@log.add_appenders(rolling_file)

        # logging of the 'logging' framework itself. Useful for debugging 'log by email' issues
        ::Logging::Logger[::Logging].level=:info
        ::Logging::Logger[::Logging].add_appenders(rolling_file)

        # unless in production or test
        unless (ENV['MAESTRO_ENV'] == 'production' || ENV['MAESTRO_ENV'] == 'test')

          # show "debug" or higher messages on STDOUT using the Basic layout
          stdout = ::Logging.appenders.stdout(
            :level => :debug,
            :layout => ::Logging::Layouts::Pattern.new(:date_pattern => date_pattern, :pattern => log_pattern))

          @@log.add_appenders(stdout)
          ::Logging::Logger[::Logging].add_appenders(stdout)

          # For some reason not having this line causes "warning: syswrite for buffered IO" messages
          # in the Logging gem.
          puts "Logging configured"
        end
      end

      def setup_logging_email(config)
        # send high priority logs via e-mail
        # options from logging https://github.com/TwP/logging/blob/master/lib/logging/appenders/email.rb
        # and net/smtp http://www.ruby-doc.org/stdlib-1.9.3/libdoc/net/smtp/rdoc/Net/SMTP.html#method-i-start

        # remove nils in config passed, change keys to be symbols instead of strings
        myconfig = config.delete_if { |k, v| v.nil? }.inject({}) { |h, (k, v)| h[k.to_sym] = v; h }
        # merge defaults with config values
        email_config = DEFAULT_EMAIL_CONFIG.merge(myconfig)
        email = ::Logging.appenders.email('email', email_config)
        @@log.add_appenders(email)
      end

      def log
        return @@log
      end

      def log_level=(level)
        @@log.level = level.is_a?(Symbol) ? level : level.downcase.to_sym
      end

      def log_level
        @@log.level
      end

      def log_path
        @@log_path
      end
    end
  end
end
